@@ -1,21 +1,20 @@
-import re
-from os import path
-from base64 import (b32encode, b64decode, b64encode, urlsafe_b64decode,
-                    urlsafe_b64encode)
+
+from base64 import b64decode, b64encode, urlsafe_b64decode, urlsafe_b64encode
 from binascii import hexlify, unhexlify
+from concurrent.futures import ThreadPoolExecutor
+from functools import wraps
 from hashlib import md5 as _md5
 from hashlib import sha1 as _sha1
 from hashlib import sha256 as _sha256
 from hashlib import sha512 as _sha512
+from http.server import BaseHTTPRequestHandler
 from json import dumps, loads
+from os import path
 from random import choice, randint
 from string import ascii_lowercase, digits
-from struct import pack, unpack
+from traceback import format_exc
 from typing import Dict, Union
-from urllib.parse import urlparse, quote_plus, unquote_plus
-from functools import wraps
-from http.server import BaseHTTPRequestHandler
-from concurrent.futures import ThreadPoolExecutor
+from urllib.parse import quote_plus, unquote_plus, urlparse
 
 import jwt
 
@@ -29,19 +28,19 @@ class Context:
 
 class _Multier():
 
-    def __init__(self, future, timeout, retry, pool):
+    def __init__(self, future, timeout,):
         self._future = future
         self._timeout = timeout
-        self._retry = retry
-        self._pool = pool
+        self._traceback = None
 
     def __getattr__(self, name):
         if (name == 'result'):
             return self.join()
-        elif (name == 'pool'):
-            return self._pool
         elif (name == 'exception'):
-            return self._future.exception()
+            return self.join(True)
+        elif (name == 'traceback'):
+            self.join(True)
+            return self._traceback
         elif (name == 'running'):
             return self._future.running()
         elif (name == 'done'):
@@ -49,20 +48,56 @@ class _Multier():
         else:
             return self._future.__getattribute__(name)
 
-    def join(self):
+    def join(self, exceptionFlag: bool = False):
         try:
-            return self._future.result(self._timeout)
-        except Exception:
-            if (self._retry > 0):
-                self._retry -= 1
-                return self.join()
-            else:
-                self._future.result = lambda: None
+            result = self._future.result()
+            return result
+        except Exception as e:
+            if exceptionFlag:
+                self._traceback = format_exc()
+                return e
+            return None
+
+
+def retryWrapper(retry_time: int = 2):
+    def decorator(func):
+        def inner(*args, **kwargs):
+            max_retry = retry_time
+            while max_retry >= 0:
+                try:
+                    ret = func(*args, **kwargs)
+                    return ret
+                except Exception as e:
+                    max_retry -= 1
+                    if max_retry < 0:
+                        raise e
+        return inner
+    return decorator
 
 
 def Threader(number: int, timeout: int = None, retry: int = 2):
-    """
-    A simple decorator function that can decorate the function to make it multi-threaded.
+    """A simple decorator function that can decorate the function to make it multi-threaded.
+
+    Args:
+        number (int): thread number
+        timeout (int, optional): function run timeout. Defaults to None.
+        retry (int, optional): number of retries. Defaults to 2.
+
+    Example:
+        from ctfbox import Threader, random_string, random_int
+    from time import sleep
+
+    @Threader(10)
+    def exp(i: int):
+        sleep(random_int(1, 5))
+        return "%d : %s" % (i, random_string())
+
+        tasks = [exp(i) for i in range(100)] # 100 tasks
+        for task in tasks:
+            # task.result return when a task completed
+            # task is a concurrent.futures.Future with some attributes
+            # result, running ,done, exception, traceback
+            print('result: %s running: %s done: %s exception: %s' % (task.result, task.running, task.done, task.exception))
     """
     def decorator(func):
         if isinstance(number, int):
@@ -75,11 +110,10 @@ def Threader(number: int, timeout: int = None, retry: int = 2):
 
         @wraps(func)
         def wrapped(*args, **kwargs):
+            retry_func = retryWrapper(retry)(func)
             return _Multier(
-                pool.submit(func, *args, **kwargs),
+                pool.submit(retry_func, *args, **kwargs),
                 timeout,
-                retry,
-                pool,
             )
         return wrapped
     return decorator
